import random
import string

from . import calculation as calc
from . import cnpj as cnpj_module
from . import cpf as cpf_module


def cpf():
    cpf_ramdom = "".join(random.choice(string.digits) for i in range(9))
    cpf_ramdom += calc.calculate_first_digit(cpf_ramdom)
    cpf_ramdom += calc.calculate_second_digit(cpf_ramdom)
    return cpf_ramdom


def cnpj():
    cnpj_ramdom = "".join(random.choice(string.digits) for i in range(12))
    cnpj_ramdom += calc.calculate_first_digit(cnpj_ramdom)
    cnpj_ramdom += calc.calculate_second_digit(cnpj_ramdom)
    return cnpj_ramdom


def cpf_with_punctuation():
    cpf_ramdom = cpf()
    return "{}.{}.{}-{}".format(cpf_ramdom[:3], cpf_ramdom[3:6], cpf_ramdom[6:9], cpf_ramdom[9:])


def cnpj_with_punctuation():
    cnpj_ramdom = cnpj()
    return "{}.{}.{}/{}-{}".format(
        cnpj_ramdom[:2], cnpj_ramdom[2:5], cnpj_ramdom[5:8], cnpj_ramdom[8:12], cnpj_ramdom[12:]
    )



from . import calculation as calc
from . import compatible as compat


@compat.check_special_characters
def validate(cpf_number):
    """This function validates a CPF number.

    This function uses calculation package to calculate both digits
    and then validates the number.

    :param cpf_number: a CPF number to be validated.  Only numbers.
    :type cpf_number: string
    :return: Bool -- True for a valid number, False otherwise.

    """

    _cpf = compat.clear_punctuation(cpf_number)

    if len(_cpf) != 11 or len(set(_cpf)) == 1:
        return False

    first_part = _cpf[:9]
    second_part = _cpf[:10]
    first_digit = _cpf[9]
    second_digit = _cpf[10]

    if first_digit == calc.calculate_first_digit(
        first_part
    ) and second_digit == calc.calculate_second_digit(second_part):
        return True

    return False



DIVISOR = 11

CPF_WEIGHTS = ((10, 9, 8, 7, 6, 5, 4, 3, 2),
              (11, 10, 9, 8, 7, 6, 5, 4, 3, 2))
CNPJ_WEIGHTS = ((5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2),
               (6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2))



def calculate_first_digit(number):
    """ This function calculates the first check digit of a
        cpf or cnpj.

        :param number: cpf (length 9) or cnpf (length 12) 
            string to check the first digit. Only numbers.
        :type number: string
        :returns: string -- the first digit

    """

    sum = 0
    if len(number) == 9:
        weights = CPF_WEIGHTS[0]
    else:
        weights = CNPJ_WEIGHTS[0]

    for i in range(len(number)):
        sum = sum + int(number[i]) * weights[i]
    rest_division = sum % DIVISOR
    if rest_division < 2:
        return '0'
    return str(11 - rest_division)


def calculate_second_digit(number):
    """ This function calculates the second check digit of
        a cpf or cnpj.

        **This function must be called after the above.**

        :param number: cpf (length 10) or cnpj 
            (length 13) number with the first digit. Only numbers.
        :type number: string
        :returns: string -- the second digit

    """

    sum = 0
    if len(number) == 10:
        weights = CPF_WEIGHTS[1]
    else:
        weights = CNPJ_WEIGHTS[1]

    for i in range(len(number)):
        sum = sum + int(number[i]) * weights[i]
    rest_division = sum % DIVISOR
    if rest_division < 2:
        return '0'
    return str(11 - rest_division)



def check_special_characters(func):
    def wrapper(document):
        not_digit = [i for i in clear_punctuation(document) if not i.isdigit()]
        return False if not_digit else func(document)

    return wrapper


def clear_punctuation(document):
    """Remove from document all pontuation signals."""
    return document.translate(str.maketrans({".": None, "-": None, "/": None}))



from . import cnpj, cpf
from .compatible import clear_punctuation


def validate(number):
    """This functions acts like a Facade to the other modules cpf and cnpj
    and validates either CPF and CNPJ numbers.
    Feel free to use this or the other modules directly.

    :param number: a CPF or CNPJ number. Clear number to have only numbers.
    :type number: string
    :return: Bool -- True if number is a valid CPF or CNPJ number.
             False if it is not or do not complain
             with the right size of these numbers.

    """
    clean_number = clear_punctuation(number)

    if len(clean_number) == 11:
        return cpf.validate(clean_number)
    elif len(clean_number) == 14:
        return cnpj.validate(clean_number)
    return False
